/*
 * https://github.com/SCR-IR/tarikh-npm
 */

var HijriDate = {};

const COUNTRY = 'IR';

HijriDate.toHijri = function (year, month, day) {
    [year, month, day] = gregorian_to_hijri(parseInt(year), parseInt(month), parseInt(day));
    return {year, month, day};
};

HijriDate.fromHijri = function (year, month, day) {
    [year, month, day] = hijri_to_gregorian(parseInt(year), parseInt(month), parseInt(day));
    return {year, month, day};
};


function gregorian_to_hijri(gY, gM, gD) {
    return julianDay_to_hijri(gregorian_to_julianDay(gY, gM, gD));
}

function hijri_to_gregorian(iY, iM, iD) {
    return julianDay_to_gregorian(hijri_to_julianDay(iY, iM, iD));
}


function gregorian_to_julianDay(gY, gM, gD) {
    var gDoM, gY2, julianDay;
    gDoM = [0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334];
    gY2 = (gM > 2) ? (gY + 1) : gY;
    julianDay = 1721059 + (365 * gY) + ~~((gY2 + 3) / 4) - ~~((gY2 + 99) / 100) + ~~((gY2 + 399) / 400) + gD + gDoM[gM - 1];
    /* 1721059 = gregorian_to_julianDay(0, 1, 1) - 1 */
    return julianDay;
}

function julianDay_to_gregorian(julianDay) {
    var days, gD, gDoM, gM, gY;
    days = -~~(1721060 - julianDay);
    gY = 400 * ~~(days / 146097);
    days %= 146097;
    if (days > 36524) {
        gY += 100 * ~~(--days / 36524);
        days %= 36524;
        if (days >= 365) {
            days++;
        }
    }
    gY += 4 * ~~(days / 1461);
    days %= 1461;
    if (days > 365) {
        gY += ~~((days - 1) / 365);
        days = (days - 1) % 365;
    }
    gD = days + 1;
    gDoM = [0, 31, ((gY % 4 === 0 && gY % 100 !== 0) || (gY % 400 === 0)) ? 29 : 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
    for (gM = 0; gM < 13 && gD > gDoM[gM]; gM++) {
        gD -= gDoM[gM];
    }
    return [gY, gM, gD];
}


function hijriA_to_julianDay(iy, im, id) {
    iy += 990;
    return ~~(id + ~~((29.5 * (im - 1)) + 0.5) + ((iy - 1) * 354) + ~~((3 + (iy * 11)) / 30) + 1597616);
}

function julianDay_to_hijriA(julianDay) {
    var id, im, iy, tmp;
    julianDay = ~~(julianDay) + 350822.5;// 350823d=990y
    iy = ~~(((30 * (julianDay - 1948439.5)) + 10646) / 10631);
    tmp = julianDay - (1948439.5 + ((iy - 1) * 354) + ~~((3 + (11 * iy)) / 30));
    iy -= 990;
    im = ~~(((tmp - 29) / 29.5) + 1.99);
    if (im > 12) {
        im = 12;
    }
    id = 1 + tmp - ~~((29.5 * (im - 1)) + 0.5);
    return [iy, im, id];
}


function hijri_to_julianDay(iY, iM, iD) {
    const HILAL = hilalIM(COUNTRY);
    if (iY < HILAL.startYear || iY > HILAL.endYear) {
        return hijriA_to_julianDay(iY, iM, iD);
    }
    let julianDay = HILAL.startJD - 1 + iD;
    for (let y in HILAL.iDoM) {
        if (y < iY) {
            julianDay += HILAL.iDoM[y][0];
        } else {
            for (let m = 1; m < iM; m++) {
                julianDay += HILAL.iDoM[iY][m];
            }
            break;
        }
    }
    return julianDay;
}

function julianDay_to_hijri(julianDay) {
    const HILAL = hilalIM(COUNTRY);
    if (julianDay < HILAL.startJD || julianDay > HILAL.endJD) {
        return julianDay_to_hijriA(julianDay);
    }
    let iM, iY;
    let iD = julianDay - HILAL.startJD + 1;
    for (iY in HILAL.iDoM) {
        if (iD > HILAL.iDoM[iY][0]) {
            iD -= HILAL.iDoM[iY][0];
        } else {
            for (iM = 1; iM < 13 && iD > HILAL.iDoM[iY][iM]; iM++) {
                iD -= HILAL.iDoM[iY][iM];
            }
            break;
        }
    }
    return [+iY, iM, ~~iD];
}


function hilalIM(country = 'IR') {
    return {
        IR: {
            startYear: 1427, /* =iDoM:firstYear */
            startJD: 2453767, /* =hijriA_to_julianDay(startYear,1,1) */

            endYear: 1443, /* =iDoM:lastYear */
            endJD: 2459790, /* =hijriA_to_julianDay(endYear+1,1,1)-1 */

            iDoM: {
                1427: [355, 30, 29, 29, 30, 29, 30, 30, 30, 30, 29, 29, 30],
                1428: [354, 29, 30, 29, 29, 29, 30, 30, 29, 30, 30, 30, 29],
                1429: [354, 30, 29, 30, 29, 29, 29, 30, 30, 29, 30, 30, 29],
                1430: [354, 30, 30, 29, 29, 30, 29, 30, 29, 29, 30, 30, 29],
                1431: [354, 30, 30, 29, 30, 29, 30, 29, 30, 29, 29, 30, 29],
                1432: [355, 30, 30, 29, 30, 30, 30, 29, 29, 30, 29, 30, 29],
                1433: [355, 29, 30, 29, 30, 30, 30, 29, 30, 29, 30, 29, 30],
                1434: [354, 29, 29, 30, 29, 30, 30, 29, 30, 30, 29, 30, 29],
                1435: [355, 29, 30, 29, 30, 29, 30, 29, 30, 30, 30, 29, 30],
                1436: [354, 29, 30, 29, 29, 30, 29, 30, 29, 30, 29, 30, 30],
                1437: [354, 29, 30, 30, 29, 30, 29, 29, 30, 29, 29, 30, 30],
                1438: [354, 29, 30, 30, 30, 29, 30, 29, 29, 30, 29, 29, 30],
                1439: [354, 29, 30, 30, 30, 30, 29, 30, 29, 29, 30, 29, 29],
                1440: [355, 30, 29, 30, 30, 30, 29, 30, 30, 29, 29, 30, 29],
                1441: [355, 29, 30, 29, 30, 30, 29, 30, 30, 29, 30, 29, 30],
                1442: [354, 29, 29, 30, 29, 30, 29, 30, 30, 29, 30, 30, 29],
                1443: [354/* |355*/, 29, 30, 30, 29, 29, 30, 29, 29, 30, 30, 30, 29/* |30 :Delta*/],
                /*
                  Delta = endJD - hijriA_to_julianDay(endYear,12,29)
                */
            }
        }
    }[country];
}
